# Test if lit_config.per_test_coverage in lit.cfg sets individual test case coverage.

# RUN: %{lit} -a -vv -Dexecute_external=False \
# RUN:     %{inputs}/per-test-coverage-by-lit-cfg/per-test-coverage-by-lit-cfg.py | \
# RUN:   FileCheck -DOUT=stdout %s

# RUN: %{lit} -a -vv -Dexecute_external=True \
# RUN:     %{inputs}/per-test-coverage-by-lit-cfg/per-test-coverage-by-lit-cfg.py | \
# RUN:   FileCheck -DOUT=stderr %s

#      CHECK: {{^}}PASS: per-test-coverage-by-lit-cfg :: per-test-coverage-by-lit-cfg.py ({{[^)]*}})
#      CHECK: Command Output ([[OUT]]):
# CHECK-NEXT: --
#      CHECK: export
#      CHECK: LLVM_PROFILE_FILE=per-test-coverage-by-lit-cfg0.profraw
#      CHECK: per-test-coverage-by-lit-cfg.py
#      CHECK: {{RUN}}: at line 2
#      CHECK: export
#      CHECK: LLVM_PROFILE_FILE=per-test-coverage-by-lit-cfg1.profraw
#      CHECK: per-test-coverage-by-lit-cfg.py
#      CHECK: {{RUN}}: at line 3
#      CHECK: export
#      CHECK: LLVM_PROFILE_FILE=per-test-coverage-by-lit-cfg2.profraw
#      CHECK: per-test-coverage-by-lit-cfg.py



# Check the env command

# RUN: not %{lit} -a -v %{inputs}/shtest-env \
# RUN: | FileCheck -match-full-lines %s
#
# END.

# Make sure env commands are included in printed commands.

# CHECK: -- Testing: 16 tests{{.*}}

# CHECK: FAIL: shtest-env :: env-args-last-is-assign.txt ({{[^)]*}})
# CHECK: env FOO=1
# CHECK: # executed command: env FOO=1
# CHECK: # | Error: 'env' requires a subcommand
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-env :: env-args-last-is-u-arg.txt ({{[^)]*}})
# CHECK: env -u FOO
# CHECK: # executed command: env -u FOO
# CHECK: # | Error: 'env' requires a subcommand
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-env :: env-args-last-is-u.txt ({{[^)]*}})
# CHECK: env -u
# CHECK: # executed command: env -u
# CHECK: # | Error: 'env' requires a subcommand
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-env :: env-args-nested-none.txt ({{[^)]*}})
# CHECK: env env env
# CHECK: # executed command: env env env
# CHECK: # | Error: 'env' requires a subcommand
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-env :: env-args-none.txt ({{[^)]*}})
# CHECK: env
# CHECK: # executed command: env
# CHECK: # | Error: 'env' requires a subcommand
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-env :: env-calls-cd.txt ({{[^)]*}})
# CHECK: env -u FOO BAR=3 cd foobar
# CHECK: # executed command: env -u FOO BAR=3 cd foobar
# CHECK: # | Error: 'env' cannot call 'cd'
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-env :: env-calls-colon.txt ({{[^)]*}})
# CHECK: env -u FOO BAR=3 :
# CHECK: # executed command: env -u FOO BAR=3 :
# CHECK: # | Error: 'env' cannot call ':'
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-env :: env-calls-echo.txt ({{[^)]*}})
# CHECK: env -u FOO BAR=3 echo hello world
# CHECK: # executed command: env -u FOO BAR=3 echo hello world
# CHECK: # | Error: 'env' cannot call 'echo'
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: PASS: shtest-env :: env-calls-env.txt ({{[^)]*}})
# CHECK: env env [[PYTHON:.+]] print_environment.py | {{.*}}
# CHECK: # executed command: env env [[PYTHON_BARE:.+]] print_environment.py
# CHECK: env FOO=2 env BAR=1 [[PYTHON]] print_environment.py | {{.*}}
# CHECK: # executed command: env FOO=2 env BAR=1 [[PYTHON_BARE]] print_environment.py
# CHECK: env -u FOO env -u BAR [[PYTHON]] print_environment.py | {{.*}}
# CHECK: # executed command: env -u FOO env -u BAR [[PYTHON_BARE]] print_environment.py
# CHECK: env -u FOO BAR=1 env -u BAR FOO=2 [[PYTHON]] print_environment.py | {{.*}}
# CHECK: # executed command: env -u FOO BAR=1 env -u BAR FOO=2 [[PYTHON_BARE]] print_environment.py
# CHECK: env -u FOO BAR=1 env -u BAR FOO=2 env BAZ=3 [[PYTHON]] print_environment.py | {{.*}}
# CHECK: # executed command: env -u FOO BAR=1 env -u BAR FOO=2 env BAZ=3 [[PYTHON_BARE]] print_environment.py
# CHECK-NOT: {{^[^#]}}
# CHECK: --

# CHECK: FAIL: shtest-env :: env-calls-export.txt ({{[^)]*}})
# CHECK: env -u FOO BAR=3 export BAZ=3
# CHECK: # executed command: env -u FOO BAR=3 export BAZ=3
# CHECK: # | Error: 'env' cannot call 'export'
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-env :: env-calls-mkdir.txt ({{[^)]*}})
# CHECK: env -u FOO BAR=3 mkdir foobar
# CHECK: # executed command: env -u FOO BAR=3 mkdir foobar
# CHECK: # | Error: 'env' cannot call 'mkdir'
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-env :: env-calls-not-builtin.txt ({{[^)]*}})
# CHECK: env -u FOO BAR=3 not rm {{.+}}.no-such-file
# CHECK: # executed command: env -u FOO BAR=3 not rm {{.+}}.no-such-file{{.*}}
# CHECK: # | Error: 'env' cannot call 'rm'
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-env :: env-calls-rm.txt ({{[^)]*}})
# CHECK: env -u FOO BAR=3 rm foobar
# CHECK: # executed command: env -u FOO BAR=3 rm foobar
# CHECK: # | Error: 'env' cannot call 'rm'
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: PASS: shtest-env :: env-u.txt ({{[^)]*}})
# CHECK: [[PYTHON]] print_environment.py | {{.*}}
# CHECK: env -u FOO [[PYTHON]] print_environment.py | {{.*}}
# CHECK: # executed command: env -u FOO [[PYTHON_BARE]] print_environment.py
# CHECK: env -u FOO -u BAR [[PYTHON]] print_environment.py | {{.*}}
# CHECK: # executed command: env -u FOO -u BAR [[PYTHON_BARE]] print_environment.py
# CHECK-NOT: {{^[^#]}}
# CHECK: --

# CHECK: PASS: shtest-env :: env.txt ({{[^)]*}})
# CHECK: env A_FOO=999 [[PYTHON]] print_environment.py | {{.*}}
# CHECK: # executed command: env A_FOO=999 [[PYTHON_BARE]] print_environment.py
# CHECK: env A_FOO=1 B_BAR=2 C_OOF=3 [[PYTHON]] print_environment.py | {{.*}}
# CHECK: # executed command: env A_FOO=1 B_BAR=2 C_OOF=3 [[PYTHON_BARE]] print_environment.py
# CHECK-NOT: {{^[^#]}}
# CHECK: --

# CHECK: PASS: shtest-env :: mixed.txt ({{[^)]*}})
# CHECK: env A_FOO=999 -u FOO [[PYTHON]] print_environment.py | {{.*}}
# CHECK: # executed command: env A_FOO=999 -u FOO [[PYTHON_BARE]] print_environment.py
# CHECK: env A_FOO=1 -u FOO B_BAR=2 -u BAR C_OOF=3 [[PYTHON]] print_environment.py | {{.*}}
# CHECK: # executed command: env A_FOO=1 -u FOO B_BAR=2 -u BAR C_OOF=3 [[PYTHON_BARE]] print_environment.py
# CHECK-NOT: {{^[^#]}}
# CHECK: --

# CHECK: Total Discovered Tests: 16
# CHECK: Passed:  4 {{\([0-9]*\.[0-9]*%\)}}
# CHECK: Failed: 12 {{\([0-9]*\.[0-9]*%\)}}
# CHECK-NOT: {{.}}



# Check that -a/-v/-vv makes the line number of the failing RUN command clear.


# RUN: not %{lit} -a %{inputs}/shtest-run-at-line | %{filter-lit} | FileCheck %s
# RUN: not %{lit} -v %{inputs}/shtest-run-at-line | %{filter-lit} | FileCheck %s
# RUN: not %{lit} -vv %{inputs}/shtest-run-at-line | %{filter-lit} | FileCheck %s
# END.


# CHECK: Testing: 8 tests


# In the case of the external shell, we check for only RUN lines in stderr in
# case some shell implementations format "set -x" output differently.

# CHECK-LABEL: FAIL: shtest-run-at-line :: external-shell/basic.txt

#       CHECK: Command Output (stderr)
#  CHECK-NEXT: --
#  CHECK-NEXT: {{^}}RUN: at line 4: true{{$}}
#  CHECK-NEXT: true
#  CHECK-NEXT: {{^}}RUN: at line 5: false{{$}}
#  CHECK-NEXT: false
# CHECK-EMPTY:
#  CHECK-NEXT: --

# CHECK-LABEL: FAIL: shtest-run-at-line :: external-shell/empty-run-line.txt

#       CHECK: Command Output (stderr)
#  CHECK-NEXT: --
#  CHECK-NEXT: {{^}}RUN: at line 2 has no command after substitutions{{$}}
#  CHECK-NEXT: {{^}}RUN: at line 3: false{{$}}
#  CHECK-NEXT: false
# CHECK-EMPTY:
#  CHECK-NEXT: --

# CHECK-LABEL: FAIL: shtest-run-at-line :: external-shell/line-continuation.txt

# The execution trace from an external sh-like shell might print the commands
# from a pipeline in any order, so this time just check that lit suppresses the
# trace of the echo command for each 'RUN: at line N: cmd-line'.

#       CHECK: Command Output (stderr)
#  CHECK-NEXT: --
#  CHECK-NEXT: {{^}}RUN: at line 4: echo 'foo bar' | FileCheck
#   CHECK-NOT: RUN
#       CHECK: {{^}}RUN: at line 6: echo 'foo baz' | FileCheck
#   CHECK-NOT: RUN
#       CHECK: --

# CHECK-LABEL: FAIL: shtest-run-at-line :: external-shell/run-line-with-newline.txt

#      CHECK: Command Output (stderr)
# CHECK-NEXT: --
# CHECK-NEXT: {{^}}RUN: at line 1: echo abc |
# CHECK-NEXT: FileCheck {{.*}} &&
# CHECK-NEXT: false
#  CHECK-NOT: RUN


# CHECK-LABEL: FAIL: shtest-run-at-line :: internal-shell/basic.txt

# CHECK:      Command Output (stdout)
# CHECK-NEXT: --
# CHECK-NEXT: # RUN: at line 1
# CHECK-NEXT: true
# CHECK-NEXT: # executed command: true
# CHECK-NEXT: # RUN: at line 2
# CHECK-NEXT: false
# CHECK-NEXT: # executed command: false
# CHECK-NOT:  RUN

# CHECK-LABEL: FAIL: shtest-run-at-line :: internal-shell/empty-run-line.txt

#      CHECK: Command Output (stdout)
# CHECK-NEXT: --
# CHECK-NEXT: # RUN: at line 2 has no command after substitutions
# CHECK-NEXT: # RUN: at line 3
# CHECK-NEXT: false
# CHECK-NEXT: # executed command: false
#  CHECK-NOT: RUN

# CHECK-LABEL: FAIL: shtest-run-at-line :: internal-shell/line-continuation.txt

# CHECK:      Command Output (stdout)
# CHECK-NEXT: --
# CHECK-NEXT: # RUN: at line 1
# CHECK-NEXT: : first line continued to second line
# CHECK-NEXT: # executed command: : first line continued to second line
# CHECK-NEXT: # RUN: at line 3
# CHECK-NEXT: echo 'foo bar' | FileCheck {{.*}}
# CHECK-NEXT: # executed command: echo 'foo bar'
# CHECK-NEXT: # executed command: FileCheck {{.*}}
# CHECK-NEXT: # RUN: at line 5
# CHECK-NEXT: echo 'foo baz' | FileCheck {{.*}}
# CHECK-NEXT: # executed command: echo 'foo baz'
# CHECK-NEXT: # executed command: FileCheck {{.*}}
# CHECK-NOT:  RUN

# CHECK-LABEL: FAIL: shtest-run-at-line :: internal-shell/run-line-with-newline.txt

#      CHECK: Command Output (stdout)
# CHECK-NEXT: --
# CHECK-NEXT: # RUN: at line 1
# CHECK-NEXT: echo abc |
# CHECK-NEXT: FileCheck {{.*}} &&
# CHECK-NEXT: false
# CHECK-NEXT: # executed command: echo abc
# CHECK-NEXT: # executed command: FileCheck {{.*}}
# CHECK-NEXT: # executed command: false
#  CHECK-NOT: RUN



# RUN: %{python} %s

import unittest

from lit.ShUtil import Command, Pipeline, Seq, ShLexer, ShParser


class TestShLexer(unittest.TestCase):
    def lex(self, str, *args, **kwargs):
        return list(ShLexer(str, *args, **kwargs).lex())

    def test_basic(self):
        self.assertEqual(
            self.lex("a|b>c&d<e;f"),
            ["a", ("|",), "b", (">",), "c", ("&",), "d", ("<",), "e", (";",), "f"],
        )

    def test_redirection_tokens(self):
        self.assertEqual(self.lex("a2>c"), ["a2", (">",), "c"])
        self.assertEqual(self.lex("a 2>c"), ["a", (">", 2), "c"])

    def test_quoting(self):
        self.assertEqual(self.lex(""" 'a' """), ["a"])
        self.assertEqual(self.lex(""" "hello\\"world" """), ['hello"world'])
        self.assertEqual(self.lex(""" "hello\\'world" """), ["hello\\'world"])
        self.assertEqual(self.lex(""" "hello\\\\world" """), ["hello\\world"])
        self.assertEqual(self.lex(""" he"llo wo"rld """), ["hello world"])
        self.assertEqual(self.lex(""" a\\ b a\\\\b """), ["a b", "a\\b"])
        self.assertEqual(self.lex(""" "" "" """), ["", ""])
        self.assertEqual(self.lex(""" a\\ b """, win32Escapes=True), ["a\\", "b"])


class TestShParse(unittest.TestCase):
    def parse(self, str):
        return ShParser(str).parse()

    def test_basic(self):
        self.assertEqual(
            self.parse("echo hello"), Pipeline([Command(["echo", "hello"], [])], False)
        )
        self.assertEqual(
            self.parse('echo ""'), Pipeline([Command(["echo", ""], [])], False)
        )
        self.assertEqual(
            self.parse("""echo -DFOO='a'"""),
            Pipeline([Command(["echo", "-DFOO=a"], [])], False),
        )
        self.assertEqual(
            self.parse('echo -DFOO="a"'),
            Pipeline([Command(["echo", "-DFOO=a"], [])], False),
        )

    def test_redirection(self):
        self.assertEqual(
            self.parse("echo hello > c"),
            Pipeline([Command(["echo", "hello"], [(((">"),), "c")])], False),
        )
        self.assertEqual(
            self.parse("echo hello > c >> d"),
            Pipeline(
                [Command(["echo", "hello"], [((">",), "c"), ((">>",), "d")])], False
            ),
        )
        self.assertEqual(
            self.parse("a 2>&1"), Pipeline([Command(["a"], [((">&", 2), "1")])], False)
        )

    def test_pipeline(self):
        self.assertEqual(
            self.parse("a | b"),
            Pipeline([Command(["a"], []), Command(["b"], [])], False),
        )

        self.assertEqual(
            self.parse("a | b | c"),
            Pipeline(
                [Command(["a"], []), Command(["b"], []), Command(["c"], [])], False
            ),
        )

    def test_list(self):
        self.assertEqual(
            self.parse("a ; b"),
            Seq(
                Pipeline([Command(["a"], [])], False),
                ";",
                Pipeline([Command(["b"], [])], False),
            ),
        )

        self.assertEqual(
            self.parse("a & b"),
            Seq(
                Pipeline([Command(["a"], [])], False),
                "&",
                Pipeline([Command(["b"], [])], False),
            ),
        )

        self.assertEqual(
            self.parse("a && b"),
            Seq(
                Pipeline([Command(["a"], [])], False),
                "&&",
                Pipeline([Command(["b"], [])], False),
            ),
        )

        self.assertEqual(
            self.parse("a || b"),
            Seq(
                Pipeline([Command(["a"], [])], False),
                "||",
                Pipeline([Command(["b"], [])], False),
            ),
        )

        self.assertEqual(
            self.parse("a && b || c"),
            Seq(
                Seq(
                    Pipeline([Command(["a"], [])], False),
                    "&&",
                    Pipeline([Command(["b"], [])], False),
                ),
                "||",
                Pipeline([Command(["c"], [])], False),
            ),
        )

        self.assertEqual(
            self.parse("a; b"),
            Seq(
                Pipeline([Command(["a"], [])], False),
                ";",
                Pipeline([Command(["b"], [])], False),
            ),
        )


if __name__ == "__main__":
    unittest.main()



# Check the pushd and popd commands

# RUN: not %{lit} -a -v %{inputs}/shtest-pushd-popd \
# RUN: | FileCheck -match-full-lines %s
#
# END.

# CHECK: -- Testing: 4 tests{{.*}}

# CHECK: FAIL: shtest-pushd-popd :: popd-args.txt ({{[^)]*}})
# CHECK: popd invalid
# CHECK: # | 'popd' does not support arguments

# CHECK: FAIL: shtest-pushd-popd :: popd-no-stack.txt ({{[^)]*}})
# CHECK: popd
# CHECK: # | popd: directory stack empty

# CHECK: FAIL: shtest-pushd-popd :: pushd-too-many-args.txt ({{[^)]*}})
# CHECK: pushd a b
# CHECK: # | 'pushd' supports only one argument

# CHECK: Total Discovered Tests: 4
# CHECK: Passed:  1 {{\([0-9]*\.[0-9]*%\)}}
# CHECK: Failed:  3 {{\([0-9]*\.[0-9]*%\)}}
# CHECK-NOT: {{.}}



# Test LLVM_PROFILE_FILE is set when --per-test-coverage is passed to command line.

# RUN: %{lit} -a -vv --per-test-coverage -Dexecute_external=False \
# RUN:     %{inputs}/per-test-coverage/per-test-coverage.py | \
# RUN:   FileCheck -DOUT=stdout %s

# RUN: %{lit} -a -vv --per-test-coverage -Dexecute_external=True \
# RUN:        %{inputs}/per-test-coverage/per-test-coverage.py | \
# RUN:   FileCheck -DOUT=stderr %s

#      CHECK: {{^}}PASS: per-test-coverage :: per-test-coverage.py ({{[^)]*}})
#      CHECK: Command Output ([[OUT]]):
# CHECK-NEXT: --
#      CHECK: export
#      CHECK: LLVM_PROFILE_FILE=per-test-coverage0.profraw
#      CHECK: per-test-coverage.py
#      CHECK: {{RUN}}: at line 2
#      CHECK: export
#      CHECK: LLVM_PROFILE_FILE=per-test-coverage1.profraw
#      CHECK: per-test-coverage.py
#      CHECK: {{RUN}}: at line 3
#      CHECK: export
#      CHECK: LLVM_PROFILE_FILE=per-test-coverage2.profraw
#      CHECK: per-test-coverage.py



# Check the internal shell handling component of the ShTest format.

# RUN: not %{lit} -v %{inputs}/shtest-shell > %t.out
# FIXME: Temporarily dump test output so we can debug failing tests on
# buildbots.
# RUN: cat %t.out
# RUN: FileCheck --input-file %t.out %s
#
# Test again in non-UTF shell to catch potential errors with python 2 seen
# on stdout-encoding.txt
# RUN: env PYTHONIOENCODING=ascii not %{lit} -a %{inputs}/shtest-shell > %t.ascii.out
# FIXME: Temporarily dump test output so we can debug failing tests on
# buildbots.
# RUN: cat %t.ascii.out
# RUN: FileCheck --input-file %t.ascii.out %s
#
# END.

# CHECK: -- Testing:

# CHECK: FAIL: shtest-shell :: cat-error-0.txt
# CHECK: *** TEST 'shtest-shell :: cat-error-0.txt' FAILED ***
# CHECK: cat -b temp1.txt
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Unsupported: 'cat':  option -b not recognized
# CHECK: # error: command failed with exit status: 1
# CHECK: ***

# CHECK: FAIL: shtest-shell :: cat-error-1.txt
# CHECK: *** TEST 'shtest-shell :: cat-error-1.txt' FAILED ***
# CHECK: cat temp1.txt
# CHECK: # .---command stderr{{-*}}
# CHECK: # | [Errno 2] No such file or directory: 'temp1.txt'
# CHECK: # error: command failed with exit status: 1
# CHECK: ***

# CHECK: FAIL: shtest-shell :: colon-error.txt
# CHECK: *** TEST 'shtest-shell :: colon-error.txt' FAILED ***
# CHECK: :
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Unsupported: ':' cannot be part of a pipeline
# CHECK: # error: command failed with exit status: 127
# CHECK: ***

# CHECK: PASS: shtest-shell :: continuations.txt

# CHECK: PASS: shtest-shell :: dev-null.txt

#      CHECK: FAIL: shtest-shell :: diff-b.txt
#      CHECK: *** TEST 'shtest-shell :: diff-b.txt' FAILED ***
#      CHECK: diff -b {{[^"]*}}.0 {{[^"]*}}.1
#      CHECK: # .---command stdout{{-*}}
#      CHECK: # | {{.*}}1,2
# CHECK-NEXT: # |   f o o
# CHECK-NEXT: # | ! b a r
# CHECK-NEXT: # | ---
# CHECK-NEXT: # |   f o o
# CHECK-NEXT: # | ! bar
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
#      CHECK: ***


# CHECK: FAIL: shtest-shell :: diff-encodings.txt
# CHECK: *** TEST 'shtest-shell :: diff-encodings.txt' FAILED ***

#      CHECK: diff -u diff-in.bin diff-in.bin
# CHECK-NEXT: # executed command: diff -u diff-in.bin diff-in.bin
#  CHECK-NOT: error

#      CHECK: diff -u diff-in.utf16 diff-in.bin && false || true
# CHECK-NEXT: # executed command: diff -u diff-in.utf16 diff-in.bin
# CHECK-NEXT: # .---command stdout{{-*}}
# CHECK-NEXT: # | ---
# CHECK-NEXT: # | +++
# CHECK-NEXT: # | @@
# CHECK-NEXT: # | {{.f.o.o.$}}
# CHECK-NEXT: # | {{-.b.a.r.$}}
# CHECK-NEXT: # | {{\+.b.a.r.}}
# CHECK-NEXT: # | {{.b.a.z.$}}
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: true

#      CHECK: diff -u diff-in.utf8 diff-in.bin && false || true
# CHECK-NEXT: # executed command: diff -u diff-in.utf8 diff-in.bin
# CHECK-NEXT: # .---command stdout{{-*}}
# CHECK-NEXT: # | ---
# CHECK-NEXT: # | +++
# CHECK-NEXT: # | @@
# CHECK-NEXT: # | -foo
# CHECK-NEXT: # | -bar
# CHECK-NEXT: # | -baz
# CHECK-NEXT: # | {{\+.f.o.o.$}}
# CHECK-NEXT: # | {{\+.b.a.r.}}
# CHECK-NEXT: # | {{\+.b.a.z.$}}
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: true

#      CHECK: diff -u diff-in.bin diff-in.utf8 && false || true
# CHECK-NEXT: # executed command: diff -u diff-in.bin diff-in.utf8
# CHECK-NEXT: # .---command stdout{{-*}}
# CHECK-NEXT: # | ---
# CHECK-NEXT: # | +++
# CHECK-NEXT: # | @@
# CHECK-NEXT: # | {{-.f.o.o.$}}
# CHECK-NEXT: # | {{-.b.a.r.}}
# CHECK-NEXT: # | {{-.b.a.z.$}}
# CHECK-NEXT: # | +foo
# CHECK-NEXT: # | +bar
# CHECK-NEXT: # | +baz
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: true

#     CHECK: cat diff-in.bin | diff -u - diff-in.bin
# CHECK-NOT: error

#     CHECK: cat diff-in.bin | diff -u diff-in.bin -
# CHECK-NOT: error

#      CHECK: cat diff-in.bin | diff -u diff-in.utf16 - && false || true
# CHECK-NEXT: # executed command: cat diff-in.bin
# CHECK-NEXT: # executed command: diff -u diff-in.utf16 -
# CHECK-NEXT: # .---command stdout{{-*}}
# CHECK-NEXT: # | ---
# CHECK-NEXT: # | +++
# CHECK-NEXT: # | @@
# CHECK-NEXT: # | {{.f.o.o.$}}
# CHECK-NEXT: # | {{-.b.a.r.$}}
# CHECK-NEXT: # | {{\+.b.a.r.}}
# CHECK-NEXT: # | {{.b.a.z.$}}
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: true

#      CHECK: cat diff-in.bin | diff -u diff-in.utf8 - && false || true
# CHECK-NEXT: # executed command: cat diff-in.bin
# CHECK-NEXT: # executed command: diff -u diff-in.utf8 -
# CHECK-NEXT: # .---command stdout{{-*}}
# CHECK-NEXT: # | ---
# CHECK-NEXT: # | +++
# CHECK-NEXT: # | @@
# CHECK-NEXT: # | -foo
# CHECK-NEXT: # | -bar
# CHECK-NEXT: # | -baz
# CHECK-NEXT: # | {{\+.f.o.o.$}}
# CHECK-NEXT: # | {{\+.b.a.r.}}
# CHECK-NEXT: # | {{\+.b.a.z.$}}
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: true

#      CHECK: cat diff-in.bin | diff -u - diff-in.utf8 && false || true
# CHECK-NEXT: # executed command: cat diff-in.bin
# CHECK-NEXT: # executed command: diff -u - diff-in.utf8
# CHECK-NEXT: # .---command stdout{{-*}}
# CHECK-NEXT: # | ---
# CHECK-NEXT: # | +++
# CHECK-NEXT: # | @@
# CHECK-NEXT: # | {{-.f.o.o.$}}
# CHECK-NEXT: # | {{-.b.a.r.}}
# CHECK-NEXT: # | {{-.b.a.z.$}}
# CHECK-NEXT: # | +foo
# CHECK-NEXT: # | +bar
# CHECK-NEXT: # | +baz
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: true

# CHECK: false

# CHECK: ***


# CHECK: FAIL: shtest-shell :: diff-error-1.txt
# CHECK: *** TEST 'shtest-shell :: diff-error-1.txt' FAILED ***
# CHECK: diff -B temp1.txt temp2.txt
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Unsupported: 'diff': option -B not recognized
# CHECK: # error: command failed with exit status: 1
# CHECK: ***

# CHECK: FAIL: shtest-shell :: diff-error-2.txt
# CHECK: *** TEST 'shtest-shell :: diff-error-2.txt' FAILED ***
# CHECK: diff temp.txt
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Error: missing or extra operand
# CHECK: # error: command failed with exit status: 1
# CHECK: ***

# CHECK: FAIL: shtest-shell :: diff-error-3.txt
# CHECK: *** TEST 'shtest-shell :: diff-error-3.txt' FAILED ***
# CHECK: diff temp.txt temp1.txt
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Error: 'diff' command failed
# CHECK: error: command failed with exit status: 1
# CHECK: ***

#      CHECK: FAIL: shtest-shell :: diff-error-4.txt
#      CHECK: *** TEST 'shtest-shell :: diff-error-4.txt' FAILED ***
#      CHECK: Exit Code: 1
#      CHECK: # .---command stdout{{-*}}
# CHECK-NEXT: # | {{.*}}diff-error-4.txt.tmp
# CHECK-NEXT: # | {{.*}}diff-error-4.txt.tmp1
# CHECK-NEXT: # | {{\*+}}
# CHECK-NEXT: # | *** 1 ****
# CHECK-NEXT: # | ! hello-first
# CHECK-NEXT: # | --- 1 ----
# CHECK-NEXT: # | ! hello-second
# CHECK-NEXT: # `---{{-*}}
#      CHECK: ***

# CHECK: FAIL: shtest-shell :: diff-error-5.txt
# CHECK: *** TEST 'shtest-shell :: diff-error-5.txt' FAILED ***
# CHECK: diff
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Error: missing or extra operand
# CHECK: # error: command failed with exit status: 1
# CHECK: ***

# CHECK: FAIL: shtest-shell :: diff-error-6.txt
# CHECK: *** TEST 'shtest-shell :: diff-error-6.txt' FAILED ***
# CHECK: diff
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Error: missing or extra operand
# CHECK: # error: command failed with exit status: 1
# CHECK: ***


# CHECK: FAIL: shtest-shell :: diff-pipes.txt

# CHECK: *** TEST 'shtest-shell :: diff-pipes.txt' FAILED ***

# CHECK: diff {{[^ ]*}}.foo {{.*}}.foo | FileCheck {{.*}}
# CHECK-NOT: note
# CHECK-NOT: error

#      CHECK: diff -u {{.*}}.foo {{.*}}.bar | FileCheck {{.*}} && false || true
# CHECK-NEXT: # executed command: diff -u {{.+}}.foo{{.*}} {{.+}}.bar{{.*}}
# CHECK-NEXT: # note: command had no output on stdout or stderr
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: FileCheck
# CHECK-NEXT: # executed command: true

#     CHECK: cat {{.*}}.foo | diff -u - {{.*}}.foo
# CHECK-NOT: note
# CHECK-NOT: error

#     CHECK: cat {{.*}}.foo | diff -u {{.*}}.foo -
# CHECK-NOT: note
# CHECK-NOT: error

#      CHECK: cat {{.*}}.bar | diff -u {{.*}}.foo - && false || true
# CHECK-NEXT: # executed command: cat {{.+}}.bar{{.*}}
# CHECK-NEXT: # executed command: diff -u {{.+}}.foo{{.*}} -
# CHECK-NEXT: # .---command stdout{{-*}}
#      CHECK: # | @@
# CHECK-NEXT: # | -foo
# CHECK-NEXT: # | +bar
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: true

#      CHECK: cat {{.*}}.bar | diff -u - {{.*}}.foo && false || true
# CHECK-NEXT: # executed command: cat {{.+}}.bar{{.*}}
# CHECK-NEXT: # executed command: diff -u - {{.+}}.foo{{.*}}
# CHECK-NEXT: # .---command stdout{{-*}}
#      CHECK: # | @@
# CHECK-NEXT: # | -bar
# CHECK-NEXT: # | +foo
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: true

#     CHECK: cat {{.*}}.foo | diff - {{.*}}.foo | FileCheck {{.*}}
# CHECK-NOT: note
# CHECK-NOT: error

#      CHECK: cat {{.*}}.bar | diff -u {{.*}}.foo - | FileCheck {{.*}}
# CHECK-NEXT: # executed command: cat {{.+}}.bar{{.*}}
# CHECK-NEXT: # executed command: diff -u {{.+}}.foo{{.*}} -
# CHECK-NEXT: note: command had no output on stdout or stderr
# CHECK-NEXT: error: command failed with exit status: 1
# CHECK-NEXT: # executed command: FileCheck
# CHECK-NEXT: # executed command: true

# CHECK: false

# CHECK: ***


# CHECK: FAIL: shtest-shell :: diff-r-error-0.txt
# CHECK: *** TEST 'shtest-shell :: diff-r-error-0.txt' FAILED ***
# CHECK: diff -r
# CHECK: # .---command stdout{{-*}}
# CHECK: # | Only in {{.*}}dir1: dir1unique
# CHECK: # | Only in {{.*}}dir2: dir2unique
# CHECK: # error: command failed with exit status: 1

# CHECK: FAIL: shtest-shell :: diff-r-error-1.txt
# CHECK: *** TEST 'shtest-shell :: diff-r-error-1.txt' FAILED ***
# CHECK: diff -r
# CHECK: # .---command stdout{{-*}}
# CHECK: # | *** {{.*}}dir1{{.*}}subdir{{.*}}f01
# CHECK: # | --- {{.*}}dir2{{.*}}subdir{{.*}}f01
# CHECK: # | ! 12345
# CHECK: # | ! 00000
# CHECK: # error: command failed with exit status: 1

# CHECK: FAIL: shtest-shell :: diff-r-error-2.txt
# CHECK: *** TEST 'shtest-shell :: diff-r-error-2.txt' FAILED ***
# CHECK: diff -r
# CHECK: # .---command stdout{{-*}}
# CHECK: # | Only in {{.*}}dir2: extrafile
# CHECK: # error: command failed with exit status: 1

# CHECK: FAIL: shtest-shell :: diff-r-error-3.txt
# CHECK: *** TEST 'shtest-shell :: diff-r-error-3.txt' FAILED ***
# CHECK: diff -r
# CHECK: # .---command stdout{{-*}}
# CHECK: # | Only in {{.*}}dir1: extra_subdir
# CHECK: # error: command failed with exit status: 1

# CHECK: FAIL: shtest-shell :: diff-r-error-4.txt
# CHECK: *** TEST 'shtest-shell :: diff-r-error-4.txt' FAILED ***
# CHECK: diff -r
# CHECK: # .---command stdout{{-*}}
# CHECK: # | File {{.*}}dir1{{.*}}extra_subdir is a directory while file {{.*}}dir2{{.*}}extra_subdir is a regular file
# CHECK: # error: command failed with exit status: 1

# CHECK: FAIL: shtest-shell :: diff-r-error-5.txt
# CHECK: *** TEST 'shtest-shell :: diff-r-error-5.txt' FAILED ***
# CHECK: diff -r
# CHECK: # .---command stdout{{-*}}
# CHECK: # | Only in {{.*}}dir1: extra_subdir
# CHECK: # error: command failed with exit status: 1

# CHECK: FAIL: shtest-shell :: diff-r-error-6.txt
# CHECK: *** TEST 'shtest-shell :: diff-r-error-6.txt' FAILED ***
# CHECK: diff -r
# CHECK: # .---command stdout{{-*}}
# CHECK: # | File {{.*}}dir1{{.*}}extra_file is a regular empty file while file {{.*}}dir2{{.*}}extra_file is a directory
# CHECK: # error: command failed with exit status: 1

# CHECK: FAIL: shtest-shell :: diff-r-error-7.txt
# CHECK: *** TEST 'shtest-shell :: diff-r-error-7.txt' FAILED ***
# CHECK: diff -r - {{.*}}
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Error: cannot recursively compare '-'
# CHECK: # error: command failed with exit status: 1

# CHECK: FAIL: shtest-shell :: diff-r-error-8.txt
# CHECK: *** TEST 'shtest-shell :: diff-r-error-8.txt' FAILED ***
# CHECK: diff -r {{.*}} -
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Error: cannot recursively compare '-'
# CHECK: # error: command failed with exit status: 1

# CHECK: PASS: shtest-shell :: diff-r.txt


# CHECK: FAIL: shtest-shell :: diff-strip-trailing-cr.txt

# CHECK: *** TEST 'shtest-shell :: diff-strip-trailing-cr.txt' FAILED ***

#      CHECK: diff -u diff-in.dos diff-in.unix && false || true
# CHECK-NEXT: # executed command: diff -u diff-in.dos diff-in.unix
# CHECK-NEXT: # .---command stdout{{-*}}
#      CHECK: # | @@
# CHECK-NEXT: # | -In this file, the
# CHECK-NEXT: # | -sequence "\r\n"
# CHECK-NEXT: # | -terminates lines.
# CHECK-NEXT: # | +In this file, the
# CHECK-NEXT: # | +sequence "\n"
# CHECK-NEXT: # | +terminates lines.
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: true

#      CHECK: diff -u diff-in.unix diff-in.dos && false || true
# CHECK-NEXT: # executed command: diff -u diff-in.unix diff-in.dos
# CHECK-NEXT: # .---command stdout{{-*}}
#      CHECK: # | @@
# CHECK-NEXT: # | -In this file, the
# CHECK-NEXT: # | -sequence "\n"
# CHECK-NEXT: # | -terminates lines.
# CHECK-NEXT: # | +In this file, the
# CHECK-NEXT: # | +sequence "\r\n"
# CHECK-NEXT: # | +terminates lines.
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: true

#      CHECK: diff -u --strip-trailing-cr diff-in.dos diff-in.unix && false || true
# CHECK-NEXT: executed command: diff -u --strip-trailing-cr diff-in.dos diff-in.unix
# CHECK-NEXT: # .---command stdout{{-*}}
#      CHECK: # | @@
# CHECK-NEXT: # |  In this file, the
# CHECK-NEXT: # | -sequence "\r\n"
# CHECK-NEXT: # | +sequence "\n"
# CHECK-NEXT: # |  terminates lines.
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: true

#      CHECK: diff -u --strip-trailing-cr diff-in.unix diff-in.dos && false || true
# CHECK-NEXT: # executed command: diff -u --strip-trailing-cr diff-in.unix diff-in.dos
# CHECK-NEXT: # .---command stdout{{-*}}
#      CHECK: # | @@
# CHECK-NEXT: # |  In this file, the
# CHECK-NEXT: # | -sequence "\n"
# CHECK-NEXT: # | +sequence "\r\n"
# CHECK-NEXT: # |  terminates lines.
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: true

# CHECK: false

# CHECK: ***


# CHECK: FAIL: shtest-shell :: diff-unified.txt

# CHECK: *** TEST 'shtest-shell :: diff-unified.txt' FAILED ***

#      CHECK: diff -u {{.*}}.foo {{.*}}.bar && false || true
# CHECK-NEXT: # executed command: diff -u {{.+}}.foo{{.*}} {{.+}}.bar{{.*}}
# CHECK-NEXT: # .---command stdout{{-*}}
#      CHECK: # | @@ {{.*}} @@
# CHECK-NEXT: # | 3
# CHECK-NEXT: # | 4
# CHECK-NEXT: # | 5
# CHECK-NEXT: # | -6 foo
# CHECK-NEXT: # | +6 bar
# CHECK-NEXT: # | 7
# CHECK-NEXT: # | 8
# CHECK-NEXT: # | 9
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: true

#      CHECK: diff -U 2 {{.*}}.foo {{.*}}.bar && false || true
# CHECK-NEXT: # executed command: diff -U 2 {{.+}}.foo{{.*}} {{.+}}.bar{{.*}}
# CHECK-NEXT: # .---command stdout{{-*}}
#      CHECK: # | @@ {{.*}} @@
# CHECK-NEXT: # | 4
# CHECK-NEXT: # | 5
# CHECK-NEXT: # | -6 foo
# CHECK-NEXT: # | +6 bar
# CHECK-NEXT: # | 7
# CHECK-NEXT: # | 8
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: true

#      CHECK: diff -U4 {{.*}}.foo {{.*}}.bar && false || true
# CHECK-NEXT: # executed command: diff -U4 {{.+}}.foo{{.*}} {{.+}}.bar{{.*}}
# CHECK-NEXT: # .---command stdout{{-*}}
#      CHECK: # | @@ {{.*}} @@
# CHECK-NEXT: # | 2
# CHECK-NEXT: # | 3
# CHECK-NEXT: # | 4
# CHECK-NEXT: # | 5
# CHECK-NEXT: # | -6 foo
# CHECK-NEXT: # | +6 bar
# CHECK-NEXT: # | 7
# CHECK-NEXT: # | 8
# CHECK-NEXT: # | 9
# CHECK-NEXT: # | 10
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: true

#      CHECK: diff -U0 {{.*}}.foo {{.*}}.bar && false || true
# CHECK-NEXT: # executed command: diff -U0 {{.+}}.foo{{.*}} {{.+}}.bar{{.*}}
# CHECK-NEXT: # .---command stdout{{-*}}
#      CHECK: # | @@ {{.*}} @@
# CHECK-NEXT: # | -6 foo
# CHECK-NEXT: # | +6 bar
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-NEXT: # executed command: true

# CHECK: diff -U 30.1 {{.*}} {{.*}} && false || true
# CHECK: # executed command: diff -U 30.1 {{.*}} {{.*}}
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Error: invalid '-U' argument: 30.1
# CHECK: # error: command failed with exit status: 1
# CHECK: # executed command: true

# CHECK: diff -U-1 {{.*}} {{.*}} && false || true
# CHECK: # executed command: diff -U-1 {{.*}} {{.*}}
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Error: invalid '-U' argument: -1
# CHECK: # error: command failed with exit status: 1
# CHECK: # executed command: true

# CHECK: false

# CHECK: ***


#      CHECK: FAIL: shtest-shell :: diff-w.txt
#      CHECK: *** TEST 'shtest-shell :: diff-w.txt' FAILED ***
#      CHECK: diff -w {{.*}}.0 {{.*}}.1
#      CHECK: # .---command stdout{{-*}}
#      CHECK: # | {{\*+}} 1,3
# CHECK-NEXT: # |   foo
# CHECK-NEXT: # |   bar
# CHECK-NEXT: # | ! baz
# CHECK-NEXT: # | ---
# CHECK-NEXT: # |   foo
# CHECK-NEXT: # |   bar
# CHECK-NEXT: # | ! bat
# CHECK-NEXT: # `---{{-*}}
# CHECK-NEXT: # error: command failed with exit status: 1
#      CHECK: ***

# CHECK: FAIL: shtest-shell :: echo-at-redirect-stderr.txt
# CHECK: *** TEST 'shtest-shell :: echo-at-redirect-stderr.txt' FAILED ***
# CHECK: @echo 2> {{.*}}
# CHECK: # executed command: @echo
# CHECK: # .---command stderr{{-*}}
# CHECK: # | stdin and stderr redirects not supported for @echo
# CHECK: error: command failed with exit status:

# CHECK: FAIL: shtest-shell :: echo-at-redirect-stdin.txt
# CHECK: *** TEST 'shtest-shell :: echo-at-redirect-stdin.txt' FAILED ***
# CHECK: @echo < {{.*}}
# CHECK: # executed command: @echo
# CHECK: # .---command stderr{{-*}}
# CHECK: # | stdin and stderr redirects not supported for @echo
# CHECK: error: command failed with exit status:

# CHECK: FAIL: shtest-shell :: echo-redirect-stderr.txt
# CHECK: *** TEST 'shtest-shell :: echo-redirect-stderr.txt' FAILED ***
# CHECK: echo 2> {{.*}}
# CHECK: # executed command: echo
# CHECK: # .---command stderr{{-*}}
# CHECK: # | stdin and stderr redirects not supported for echo
# CHECK: error: command failed with exit status:

# CHECK: FAIL: shtest-shell :: echo-redirect-stdin.txt
# CHECK: *** TEST 'shtest-shell :: echo-redirect-stdin.txt' FAILED ***
# CHECK: echo < {{.*}}
# CHECK: # executed command: echo
# CHECK: # .---command stderr{{-*}}
# CHECK: # | stdin and stderr redirects not supported for echo
# CHECK: error: command failed with exit status:

# CHECK: FAIL: shtest-shell :: error-0.txt
# CHECK: *** TEST 'shtest-shell :: error-0.txt' FAILED ***
# CHECK: not-a-real-command
# CHECK: # .---command stderr{{-*}}
# CHECK: # | 'not-a-real-command': command not found
# CHECK: # error: command failed with exit status: 127
# CHECK: ***

# FIXME: The output here sucks.
#
#       CHECK: UNRESOLVED: shtest-shell :: error-1.txt
#  CHECK-NEXT: *** TEST 'shtest-shell :: error-1.txt' FAILED ***
#  CHECK-NEXT: Exit Code: 1
# CHECK-EMPTY:
#  CHECK-NEXT: Command Output (stdout):
#  CHECK-NEXT: --
#  CHECK-NEXT: # shell parser error on RUN: at line 3: echo "missing quote
# CHECK-EMPTY:
#  CHECK-NEXT: --
# CHECK-EMPTY:
#  CHECK-NEXT: ***

# CHECK: FAIL: shtest-shell :: error-2.txt
# CHECK: *** TEST 'shtest-shell :: error-2.txt' FAILED ***
# CHECK: Unsupported redirect:
# CHECK: ***

# CHECK: FAIL: shtest-shell :: mkdir-error-0.txt
# CHECK: *** TEST 'shtest-shell :: mkdir-error-0.txt' FAILED ***
# CHECK: mkdir -p temp | rm -rf temp
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Unsupported: 'mkdir' cannot be part of a pipeline
# CHECK: # error: command failed with exit status: 127
# CHECK: ***

# CHECK: FAIL: shtest-shell :: mkdir-error-1.txt
# CHECK: *** TEST 'shtest-shell :: mkdir-error-1.txt' FAILED ***
# CHECK: mkdir -p -m 777 temp
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Unsupported: 'mkdir': option -m not recognized
# CHECK: # error: command failed with exit status: 127
# CHECK: ***

# CHECK: FAIL: shtest-shell :: mkdir-error-2.txt
# CHECK: *** TEST 'shtest-shell :: mkdir-error-2.txt' FAILED ***
# CHECK: mkdir -p
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Error: 'mkdir' is missing an operand
# CHECK: # error: command failed with exit status: 127
# CHECK: ***

# CHECK: PASS: shtest-shell :: redirects.txt

# CHECK: FAIL: shtest-shell :: rm-error-0.txt
# CHECK: *** TEST 'shtest-shell :: rm-error-0.txt' FAILED ***
# CHECK: rm -rf temp | echo "hello"
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Unsupported: 'rm' cannot be part of a pipeline
# CHECK: # error: command failed with exit status: 127
# CHECK: ***

# CHECK: FAIL: shtest-shell :: rm-error-1.txt
# CHECK: *** TEST 'shtest-shell :: rm-error-1.txt' FAILED ***
# CHECK: rm -f -v temp
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Unsupported: 'rm': option -v not recognized
# CHECK: # error: command failed with exit status: 127
# CHECK: ***

# CHECK: FAIL: shtest-shell :: rm-error-2.txt
# CHECK: *** TEST 'shtest-shell :: rm-error-2.txt' FAILED ***
# CHECK: rm -r hello
# CHECK: # .---command stderr{{-*}}
# CHECK: # | Error: 'rm' command failed
# CHECK: # error: command failed with exit status: 1
# CHECK: ***

# CHECK: FAIL: shtest-shell :: rm-error-3.txt
# CHECK: *** TEST 'shtest-shell :: rm-error-3.txt' FAILED ***
# CHECK: Exit Code: 1
# CHECK: ***

# CHECK: PASS: shtest-shell :: rm-unicode-0.txt
# CHECK: PASS: shtest-shell :: sequencing-0.txt
# CHECK: XFAIL: shtest-shell :: sequencing-1.txt

#      CHECK: FAIL: shtest-shell :: stdout-encoding.txt
#      CHECK: *** TEST 'shtest-shell :: stdout-encoding.txt' FAILED ***
#      CHECK: cat diff-in.bin
#      CHECK: # .---command stdout{{-*}}
# CHECK-NEXT: # | {{.f.o.o.$}}
# CHECK-NEXT: # | {{.b.a.r.}}
# CHECK-NEXT: # | {{.b.a.z.$}}
# CHECK-NEXT: # `---{{-*}}
#  CHECK-NOT: error
#      CHECK: false
#      CHECK: ***

# CHECK: PASS: shtest-shell :: valid-shell.txt
# CHECK: Unresolved Tests (1)
# CHECK: Failed Tests (38)



# Check the various features of the ShTest format.
#
# RUN: not %{lit} -v %{inputs}/shtest-output-printing > %t.out
# RUN: FileCheck --input-file %t.out --match-full-lines %s
#
# END.

#       CHECK: -- Testing: {{.*}}
#       CHECK: FAIL: shtest-output-printing :: basic.txt {{.*}}
#  CHECK-NEXT: ***{{\**}} TEST 'shtest-output-printing :: basic.txt' FAILED ***{{\**}}
#  CHECK-NEXT: Exit Code: 1
# CHECK-EMPTY:
#  CHECK-NEXT: Command Output (stdout):
#  CHECK-NEXT: --
#  CHECK-NEXT: # RUN: at line 1
#  CHECK-NEXT: true
#  CHECK-NEXT: # executed command: true
#  CHECK-NEXT: # RUN: at line 2
#  CHECK-NEXT: echo hi
#  CHECK-NEXT: # executed command: echo hi
#  CHECK-NEXT: # .---command stdout------------
#  CHECK-NEXT: # | hi
#  CHECK-NEXT: # `-----------------------------
#  CHECK-NEXT: # RUN: at line 3
#  CHECK-NEXT: not not wc missing-file &> [[FILE:.*]] || true
#  CHECK-NEXT: # executed command: not not wc missing-file
#  CHECK-NEXT: # .---redirected output from '[[FILE]]'
#  CHECK-NEXT: # | wc: {{cannot open missing-file|missing-file.* No such file or directory}}
#  CHECK-NEXT: # `-----------------------------
#  CHECK-NEXT: # note: command had no output on stdout or stderr
#  CHECK-NEXT: # error: command failed with exit status: 1
#  CHECK-NEXT: # executed command: true
#  CHECK-NEXT: # RUN: at line 4
#  CHECK-NEXT: not {{.*}}python{{.*}} {{.*}}write-a-lot.py &> [[FILE:.*]]
#  CHECK-NEXT: # executed command: not {{.*}}python{{.*}} {{.*}}write-a-lot.py{{.*}}
#  CHECK-NEXT: # .---redirected output from '[[FILE]]'
#  CHECK-NEXT: # | All work and no play makes Jack a dull boy.
#  CHECK-NEXT: # | All work and no play makes Jack a dull boy.
#  CHECK-NEXT: # | All work and no play makes Jack a dull boy.
#       CHECK: # | ...
#  CHECK-NEXT: # `---data was truncated--------
#  CHECK-NEXT: # note: command had no output on stdout or stderr
#  CHECK-NEXT: # error: command failed with exit status: 1
# CHECK-EMPTY:
#  CHECK-NEXT:--



# Check the not command

# RUN: not %{lit} -a -v %{inputs}/shtest-not \
# RUN: | FileCheck -match-full-lines %s
#
# END.

# Make sure not and env commands are included in printed commands.

# CHECK: -- Testing: 17 tests{{.*}}

# CHECK: FAIL: shtest-not :: exclamation-args-nested-none.txt {{.*}}
# CHECK: ! ! !
# CHECK: # executed command: ! ! !
# CHECK: # | Error: '!' requires a subcommand
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-not :: exclamation-args-none.txt {{.*}}
# CHECK: !
# CHECK: # executed command: !
# CHECK: # | Error: '!' requires a subcommand
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-not :: exclamation-calls-external.txt {{.*}}

# CHECK: ! [[PYTHON:.*]] fail.py
# CHECK: # executed command: ! [[PYTHON_BARE:.*]] fail.py
# CHECK: ! ! [[PYTHON]] pass.py
# CHECK: # executed command: ! ! [[PYTHON_BARE]] pass.py
# CHECK: ! ! ! [[PYTHON]] fail.py
# CHECK: # executed command: ! ! ! [[PYTHON_BARE]] fail.py
# CHECK: ! ! ! ! [[PYTHON]] pass.py
# CHECK: # executed command: ! ! ! ! [[PYTHON_BARE]] pass.py

# CHECK: ! [[PYTHON]] pass.py
# CHECK: # executed command: ! [[PYTHON_BARE]] pass.py
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-not :: not-args-last-is-crash.txt {{.*}}
# CHECK: not --crash
# CHECK: # executed command: not --crash
# CHECK: # | Error: 'not' requires a subcommand
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-not :: not-args-nested-none.txt {{.*}}
# CHECK: not not not
# CHECK: # executed command: not not not
# CHECK: # | Error: 'not' requires a subcommand
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-not :: not-args-none.txt {{.*}}
# CHECK: not
# CHECK: # executed command: not
# CHECK: # | Error: 'not' requires a subcommand
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-not :: not-calls-cd.txt {{.*}}
# CHECK: not not cd foobar
# CHECK: # executed command: not not cd foobar
# CHECK: not --crash cd foobar
# CHECK: # executed command: not --crash cd foobar
# CHECK: # | Error: 'not --crash' cannot call 'cd'
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-not :: not-calls-colon.txt {{.*}}
# CHECK: not not : foobar
# CHECK: # executed command: not not : foobar
# CHECK: not --crash :
# CHECK: # executed command: not --crash :
# CHECK: # | Error: 'not --crash' cannot call ':'
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-not :: not-calls-diff-with-crash.txt {{.*}}
# CHECK: not --crash diff -u {{.*}}
# CHECK: # executed command: not --crash diff -u {{.*}}
# CHECK-NOT: # executed command: {{.*}}
# CHECK-NOT: {{[Ee]rror}}
# CHECK: # error: command failed with exit status: {{.*}}
# CHECK-NOT: # executed command: {{.*}}
# CHECK-NOT: {{[Ee]rror}}

# CHECK: FAIL: shtest-not :: not-calls-diff.txt {{.*}}
# CHECK: not diff {{.*}}
# CHECK: # executed command: not diff {{.*}}
# CHECK: not not not diff {{.*}}
# CHECK: # executed command: not not not diff {{.*}}
# CHECK: not not not not not diff {{.*}}
# CHECK: # executed command: not not not not not diff {{.*}}
# CHECK: diff {{.*}}
# CHECK: # executed command: diff {{.*}}
# CHECK: not not diff {{.*}}
# CHECK: # executed command: not not diff {{.*}}
# CHECK: not not not not diff {{.*}}
# CHECK: # executed command: not not not not diff {{.*}}
# CHECK: not diff {{.*}}
# CHECK: # executed command: not diff {{.*}}
# CHECK-NOT: # executed command: {{.*}}

# CHECK: FAIL: shtest-not :: not-calls-echo.txt {{.*}}
# CHECK: not not echo hello world
# CHECK: # executed command: not not echo hello world
# CHECK: not --crash echo hello world
# CHECK: # executed command: not --crash echo hello world
# CHECK: # | Error: 'not --crash' cannot call 'echo'
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-not :: not-calls-env-builtin.txt {{.*}}
# CHECK: not --crash env -u FOO BAR=3 rm {{.*}}.no-such-file
# CHECK: # executed command: not --crash env -u FOO BAR=3 rm {{.+}}.no-such-file{{.*}}
# CHECK: # | Error: 'env' cannot call 'rm'
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-not :: not-calls-export.txt {{.*}}
# CHECK: not not export FOO=1
# CHECK: # executed command: not not export FOO=1
# CHECK: not --crash export BAZ=3
# CHECK: # executed command: not --crash export BAZ=3
# CHECK: # | Error: 'not --crash' cannot call 'export'
# CHECK: # error: command failed with exit status: {{.*}}


# CHECK: PASS: shtest-not :: not-calls-external.txt {{.*}}

# CHECK: not [[PYTHON]] fail.py
# CHECK: # executed command: not [[PYTHON_BARE]] fail.py
# CHECK: not not [[PYTHON]] pass.py
# CHECK: # executed command: not not [[PYTHON_BARE]] pass.py
# CHECK: not not not [[PYTHON]] fail.py
# CHECK: # executed command: not not not [[PYTHON_BARE]] fail.py
# CHECK: not not not not [[PYTHON]] pass.py
# CHECK: # executed command: not not not not [[PYTHON_BARE]] pass.py

# CHECK: not not --crash [[PYTHON]] pass.py
# CHECK: # executed command: not not --crash [[PYTHON_BARE]] pass.py
# CHECK: not not --crash [[PYTHON]] fail.py
# CHECK: # executed command: not not --crash [[PYTHON_BARE]] fail.py
# CHECK: not not --crash not [[PYTHON]] pass.py
# CHECK: # executed command: not not --crash not [[PYTHON_BARE]] pass.py
# CHECK: not not --crash not [[PYTHON]] fail.py
# CHECK: # executed command: not not --crash not [[PYTHON_BARE]] fail.py

# CHECK: env not [[PYTHON]] fail.py | {{.*}}
# CHECK: # executed command: env not [[PYTHON_BARE]] fail.py
# CHECK: not env [[PYTHON]] fail.py | {{.*}}
# CHECK: # executed command: not env [[PYTHON_BARE]] fail.py
# CHECK: env FOO=1 not [[PYTHON]] fail.py | {{.*}}
# CHECK: # executed command: env FOO=1 not [[PYTHON_BARE]] fail.py
# CHECK: not env FOO=1 BAR=1 [[PYTHON]] fail.py | {{.*}}
# CHECK: # executed command: not env FOO=1 BAR=1 [[PYTHON_BARE]] fail.py
# CHECK: env FOO=1 BAR=1 not env -u FOO BAR=2 [[PYTHON]] fail.py | {{.*}}
# CHECK: # executed command: env FOO=1 BAR=1 not env -u FOO BAR=2 [[PYTHON_BARE]] fail.py
# CHECK: not env FOO=1 BAR=1 not env -u FOO -u BAR [[PYTHON]] pass.py | {{.*}}
# CHECK: # executed command: not env FOO=1 BAR=1 not env -u FOO -u BAR [[PYTHON_BARE]] pass.py
# CHECK: not not env FOO=1 env FOO=2 BAR=1 [[PYTHON]] pass.py | {{.*}}
# CHECK: # executed command: not not env FOO=1 env FOO=2 BAR=1 [[PYTHON_BARE]] pass.py
# CHECK: env FOO=1 -u BAR env -u FOO BAR=1 not not [[PYTHON]] pass.py | {{.*}}
# CHECK: # executed command: env FOO=1 -u BAR env -u FOO BAR=1 not not [[PYTHON_BARE]] pass.py

# CHECK: not env FOO=1 BAR=1 env FOO=2 BAR=2 not --crash [[PYTHON]] pass.py | {{.*}}
# CHECK: # executed command: not env FOO=1 BAR=1 env FOO=2 BAR=2 not --crash [[PYTHON_BARE]] pass.py
# CHECK: not env FOO=1 BAR=1 not --crash not [[PYTHON]] pass.py | {{.*}}
# CHECK: # executed command: not env FOO=1 BAR=1 not --crash not [[PYTHON_BARE]] pass.py
# CHECK: not not --crash env -u BAR not env -u FOO BAR=1 [[PYTHON]] pass.py | {{.*}}
# CHECK: # executed command: not not --crash env -u BAR not env -u FOO BAR=1 [[PYTHON_BARE]] pass.py


# CHECK: FAIL: shtest-not :: not-calls-fail2.txt {{.*}}
# CHECK-NEXT: {{.*}} TEST 'shtest-not :: not-calls-fail2.txt' FAILED {{.*}}
# CHECK-NEXT: Exit Code: 1

# CHECK: FAIL: shtest-not :: not-calls-mkdir.txt {{.*}}
# CHECK: not mkdir {{.*}}
# CHECK: # executed command: not mkdir {{.*}}
# CHECK: not --crash mkdir foobar
# CHECK: # executed command: not --crash mkdir foobar
# CHECK: # | Error: 'not --crash' cannot call 'mkdir'
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: FAIL: shtest-not :: not-calls-rm.txt {{.*}}
# CHECK: not rm {{.*}}
# CHECK: # executed command: not rm {{.*}}
# CHECK: not --crash rm foobar
# CHECK: # executed command: not --crash rm foobar
# CHECK: # | Error: 'not --crash' cannot call 'rm'
# CHECK: # error: command failed with exit status: {{.*}}

# CHECK: Total Discovered Tests: 17
# CHECK: Passed:  1 {{\([0-9]*\.[0-9]*%\)}}
# CHECK: Failed: 16 {{\([0-9]*\.[0-9]*%\)}}
# CHECK-NOT: {{.}}



# Check that the environment variable is set correctly
# RUN: %{python} %s | FileCheck -DINDEX=1 %s
# RUN: %{python} %s | FileCheck -DINDEX=2 %s

# Python script to read the environment variable
# and print its value
import os

llvm_profile_file = os.environ.get('LLVM_PROFILE_FILE')
print(llvm_profile_file)

# CHECK: per-test-coverage-by-lit-cfg[[INDEX]].profraw



import lit.util
import os
import sys

main_config = lit.util.abs_path_preserve_drive(sys.argv[1])
main_config = os.path.normcase(main_config)

config_map = {main_config: sys.argv[2]}
builtin_parameters = {"config_map": config_map}

if __name__ == "__main__":
    from lit.main import main

    main_config_dir = os.path.dirname(main_config)
    sys.argv = [sys.argv[0]] + sys.argv[3:] + [main_config_dir]
    main(builtin_parameters)



import os
import lit.formats

class CustomFormat(lit.formats.ShTest):
    def getTestsForPath(self, testSuite, path_in_suite, litConfig, localConfig):
        for sub in ['one.test', 'two.test']:
            basePath = os.path.dirname(testSuite.getExecPath(path_in_suite))
            os.makedirs(basePath, exist_ok=True)
            generatedFile = os.path.join(basePath, sub)
            with open(generatedFile, 'w') as f:
                f.write('RUN: true')
            yield lit.Test.Test(testSuite, (generatedFile, ), localConfig)



import lit
import lit.formats

CUSTOM_PASS = lit.Test.ResultCode("CUSTOM_PASS", "My Passed", False)
CUSTOM_FAILURE = lit.Test.ResultCode("CUSTOM_FAILURE", "My Failed", True)


class MyFormat(lit.formats.ShTest):
    def execute(self, test, lit_config):
        result = super(MyFormat, self).execute(test, lit_config)
        if result.code.isFailure:
            result.code = CUSTOM_FAILURE
        else:
            result.code = CUSTOM_PASS
        return result



# Check that we can inject preamble commands at the beginning of a ShTest.
#
# For one case, check the execution trace as these preamble commands have
# "preamble command" instead of the usual "{{RUN}}: at line N".

# RUN: %{lit} %{inputs}/shtest-inject/test-empty.txt --show-all | FileCheck --check-prefix=CHECK-TEST1 %s
#
#       CHECK-TEST1: Command Output (stdout):
#  CHECK-TEST1-NEXT: --
#  CHECK-TEST1-NEXT: # preamble command line
#  CHECK-TEST1-NEXT: echo "THIS WAS"
#  CHECK-TEST1-NEXT: # executed command: echo 'THIS WAS'
#  CHECK-TEST1-NEXT: # .---command stdout{{-*}}
#  CHECK-TEST1-NEXT: # | THIS WAS
#  CHECK-TEST1-NEXT: # `---{{-*}}
#  CHECK-TEST1-NEXT: # preamble command line
#  CHECK-TEST1-NEXT: echo
#  CHECK-TEST1-NEXT: "INJECTED"
#  CHECK-TEST1-NEXT: # executed command: echo INJECTED
#  CHECK-TEST1-NEXT: # .---command stdout{{-*}}
#  CHECK-TEST1-NEXT: # | INJECTED
#  CHECK-TEST1-NEXT: # `---{{-*}}
# CHECK-TEST1-EMPTY:
#  CHECK-TEST1-NEXT: --
#
# CHECK-TEST1: Passed: 1

# RUN: %{lit} %{inputs}/shtest-inject/test-one.txt --show-all | FileCheck --check-prefix=CHECK-TEST2 %s
#
# CHECK-TEST2: THIS WAS
# CHECK-TEST2: INJECTED
# CHECK-TEST2: IN THE FILE
#
# CHECK-TEST2: Passed: 1

# RUN: %{lit} %{inputs}/shtest-inject/test-many.txt --show-all | FileCheck --check-prefix=CHECK-TEST3 %s
#
# CHECK-TEST3: THIS WAS
# CHECK-TEST3: INJECTED
# CHECK-TEST3: IN THE FILE
# CHECK-TEST3: IF IT WORKS
# CHECK-TEST3: AS EXPECTED
#
# CHECK-TEST3: Passed: 1



import os
import sys


def execute(fileName):
    sys.stderr.write(
        "error: external '{}' command called unexpectedly\n".format(
            os.path.basename(fileName)
        )
    )
    sys.exit(1)



import os
import sys

from setuptools import setup, find_packages

# setuptools expects to be invoked from within the directory of setup.py, but it
# is nice to allow:
#   python path/to/setup.py install
# to work (for scripts, etc.)
os.chdir(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, ".")

import lit

with open("README.rst", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="lit",
    version=lit.__version__,
    author=lit.__author__,
    author_email=lit.__email__,
    url="http://llvm.org",
    license="Apache-2.0 with LLVM exception",
    license_files=["LICENSE.TXT"],
    description="A Software Testing Tool",
    keywords="test C++ automatic discovery",
    long_description=long_description,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Testing",
    ],
    zip_safe=False,
    packages=find_packages(),
    entry_points={
        "console_scripts": [
            "lit = lit.main:main",
        ],
    },
)



# Check the behavior of the ALLOW_RETRIES keyword.

# This test uses a file that's stable across retries of the test to fail and
# only succeed the fourth time it is retried.
#
# RUN: rm -f %t.counter
# RUN: %{lit} %{inputs}/allow-retries/succeeds-within-limit.py -Dcounter=%t.counter -Dpython=%{python} | FileCheck --check-prefix=CHECK-TEST1 %s
# CHECK-TEST1: Passed With Retry: 1

# Test that a per-file ALLOW_RETRIES overwrites the config-wide test_retry_attempts property, if any.
#
# RUN: rm -f %t.counter
# RUN: %{lit} %{inputs}/allow-retries/succeeds-within-limit.py -Dtest_retry_attempts=2 -Dcounter=%t.counter -Dpython=%{python} | FileCheck --check-prefix=CHECK-TEST2 %s
# CHECK-TEST2: Passed With Retry: 1

# This test does not succeed within the allowed retry limit
#
# Check that the execution trace isn't corrupt due to reprocessing the script
# multiple times (e.g., '%dbg(...)' processing used to accumulate across
# retries).
#
# RUN: not %{lit} %{inputs}/allow-retries/does-not-succeed-within-limit.py -v |\
# RUN:   FileCheck --check-prefix=CHECK-TEST3 -match-full-lines %s
#
#       CHECK-TEST3: FAIL: allow-retries :: does-not-succeed-within-limit.py (1 of 1)
#  CHECK-TEST3-NEXT: {{\**}} TEST 'allow-retries :: does-not-succeed-within-limit.py' FAILED {{\**}}
#  CHECK-TEST3-NEXT: Exit Code: 1
# CHECK-TEST3-EMPTY:
#  CHECK-TEST3-NEXT: Command Output (stdout):
#  CHECK-TEST3-NEXT: --
#  CHECK-TEST3-NEXT: # {{RUN}}: at line 3
#  CHECK-TEST3-NEXT: false
#  CHECK-TEST3-NEXT: # executed command: false
#  CHECK-TEST3-NEXT: # note: command had no output on stdout or stderr
#  CHECK-TEST3-NEXT: # error: command failed with exit status: 1
# CHECK-TEST3-EMPTY:
#  CHECK-TEST3-NEXT: --
#       CHECK-TEST3: Failed Tests (1):
#       CHECK-TEST3: allow-retries :: does-not-succeed-within-limit.py

# This test should be UNRESOLVED since it has more than one ALLOW_RETRIES
# lines, and that is not allowed.
#
# RUN: not %{lit} %{inputs}/allow-retries/more-than-one-allow-retries-lines.py | FileCheck --check-prefix=CHECK-TEST4 %s
# CHECK-TEST4: Unresolved Tests (1):
# CHECK-TEST4: allow-retries :: more-than-one-allow-retries-lines.py

# This test does not provide a valid integer to the ALLOW_RETRIES keyword.
# It should be unresolved.
#
# RUN: not %{lit} %{inputs}/allow-retries/not-a-valid-integer.py | FileCheck --check-prefix=CHECK-TEST5 %s
# CHECK-TEST5: Unresolved Tests (1):
# CHECK-TEST5: allow-retries :: not-a-valid-integer.py

# This test checks that the config-wide test_retry_attempts property is used
# when no ALLOW_RETRIES keyword is present.
#
# RUN: rm -f %t.counter
# RUN: %{lit} %{inputs}/test_retry_attempts/test.py -Dcounter=%t.counter -Dpython=%{python} | FileCheck --check-prefix=CHECK-TEST6 %s
# CHECK-TEST6: Passed With Retry: 1

# This test checks that --per-test-coverage doesn't accumulate inserted
# LLVM_PROFILE_FILE= commands across retries.
#
# RUN: rm -f %t.counter
# RUN: %{lit} -a %{inputs}/test_retry_attempts/test.py --per-test-coverage\
# RUN:     -Dcounter=%t.counter -Dpython=%{python} | \
# RUN:   FileCheck --check-prefix=CHECK-TEST7 %s
#     CHECK-TEST7: Command Output (stdout):
#     CHECK-TEST7: # executed command: export LLVM_PROFILE_FILE=
# CHECK-TEST7-NOT: # executed command: export LLVM_PROFILE_FILE=
#     CHECK-TEST7: Passed With Retry: 1



# This test exercises an external shell use case that, at least at one time,
# appeared in the following tests:
#
#   compiler-rt/test/fuzzer/fork-sigusr.test
#   compiler-rt/test/fuzzer/merge-sigusr.test
#   compiler-rt/test/fuzzer/sigint.test
#   compiler-rt/test/fuzzer/sigusr.test
#
# That is, a RUN line can be:
#
#   cmd & PID=$!
#
# It is important that '&' only puts 'cmd' in the background and not the
# debugging commands that lit inserts before 'cmd'.  Otherwise:
#
# - The debugging commands might execute later than they are supposed to.
# - A later 'kill $PID' can kill more than just 'cmd'.  We've seen it even
#   manage to terminate the shell running lit.
#
# The last FileCheck directive below checks that the debugging commands for the
# above RUN line are not killed and do execute at the right time.

# RUN: %{lit} -a %{inputs}/shtest-external-shell-kill | %{filter-lit} | FileCheck %s
# END.

#       CHECK: Command Output (stdout):
#  CHECK-NEXT: --
#  CHECK-NEXT: start
#  CHECK-NEXT: end
# CHECK-EMPTY:
#  CHECK-NEXT: --
#  CHECK-NEXT: Command Output (stderr):
#  CHECK-NEXT: --
#  CHECK-NEXT: RUN: at line 1: echo start
#  CHECK-NEXT: echo start
#  CHECK-NEXT: RUN: at line 2: sleep [[#]] & PID=$!
